@@ -3,12 +3,18 @@ Template tags for working with lists of model instances which represent
 trees.
 """
 from __future__ import unicode_literals
+import warnings
+
 from django import template
-from django.db.models import get_model
+try:
+    from django.apps import apps
+    get_model = apps.get_model
+except ImportError:  # pragma: no cover (Django 1.6 compatibility)
+    from django.db.models import get_model
 from django.db.models.fields import FieldDoesNotExist
 try:
     from django.utils.encoding import force_text
-except ImportError:
+except ImportError:  # pragma: no cover (Django 1.4 compatibility)
     from django.utils.encoding import force_unicode as force_text
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
@@ -18,7 +24,7 @@ from mptt.utils import tree_item_iterator, drilldown_tree_for_node
 register = template.Library()
 
 
-### ITERATIVE TAGS
+# ## ITERATIVE TAGS
 
 class FullTreeForModelNode(template.Node):
     def __init__(self, model, context_var):
@@ -53,7 +59,7 @@ class DrilldownTreeForNodeNode(template.Node):
             cls = get_model(app_label, model_name)
             if cls is None:
                 raise template.TemplateSyntaxError(
-                    _('drilldown_tree_for_node tag was given an invalid model: %s') % \
+                    _('drilldown_tree_for_node tag was given an invalid model: %s') %
                     '.'.join([app_label, model_name])
                 )
             try:
@@ -136,7 +142,7 @@ def do_drilldown_tree_for_node(parser, token):
        {% drilldown_tree_for_node genre as drilldown count tests.Game.genre in game_count %}
        {% drilldown_tree_for_node genre as drilldown cumulative count tests.Game.genre in game_count %}
 
-    """
+    """  # noqa
     bits = token.contents.split()
     len_bits = len(bits)
     if len_bits not in (4, 8, 9):
@@ -148,21 +154,26 @@ def do_drilldown_tree_for_node(parser, token):
     if len_bits == 8:
         if bits[4] != 'count':
             raise template.TemplateSyntaxError(
-                _("if seven arguments are given, fourth argument to %s tag must be 'with'") % bits[0])
+                _("if seven arguments are given, fourth argument to %s tag must be 'with'")
+                % bits[0])
         if bits[6] != 'in':
             raise template.TemplateSyntaxError(
-                _("if seven arguments are given, sixth argument to %s tag must be 'in'") % bits[0])
+                _("if seven arguments are given, sixth argument to %s tag must be 'in'")
+                % bits[0])
         return DrilldownTreeForNodeNode(bits[1], bits[3], bits[5], bits[7])
     elif len_bits == 9:
         if bits[4] != 'cumulative':
             raise template.TemplateSyntaxError(
-                _("if eight arguments are given, fourth argument to %s tag must be 'cumulative'") % bits[0])
+                _("if eight arguments are given, fourth argument to %s tag must be 'cumulative'")
+                % bits[0])
         if bits[5] != 'count':
             raise template.TemplateSyntaxError(
-                _("if eight arguments are given, fifth argument to %s tag must be 'count'") % bits[0])
+                _("if eight arguments are given, fifth argument to %s tag must be 'count'")
+                % bits[0])
         if bits[7] != 'in':
             raise template.TemplateSyntaxError(
-                _("if eight arguments are given, seventh argument to %s tag must be 'in'") % bits[0])
+                _("if eight arguments are given, seventh argument to %s tag must be 'in'")
+                % bits[0])
         return DrilldownTreeForNodeNode(bits[1], bits[3], bits[6], bits[8], cumulative=True)
     else:
         return DrilldownTreeForNodeNode(bits[1], bits[3])
@@ -191,9 +202,9 @@ def tree_info(items, features=None):
     Example::
 
        {% for genre,structure in genres|tree_info %}
-       {% if tree.new_level %}<ul><li>{% else %}</li><li>{% endif %}
+       {% if structure.new_level %}<ul><li>{% else %}</li><li>{% endif %}
        {{ genre.name }}
-       {% for level in tree.closed_levels %}</li></ul>{% endfor %}
+       {% for level in structure.closed_levels %}</li></ul>{% endfor %}
        {% endfor %}
 
     """
@@ -220,10 +231,10 @@ def tree_path(items, separator=' :: '):
        {{ some_node.get_ancestors|tree_path:" > " }}
 
     """
-    return separator.join([force_text(i) for i in items])
+    return separator.join(force_text(i) for i in items)
 
 
-### RECURSIVE TAGS
+# ## RECURSIVE TAGS
 
 @register.filter
 def cache_tree_children(queryset):
@@ -236,7 +247,6 @@ def cache_tree_children(queryset):
 
     Returns a list of top-level nodes. If a single tree was provided in its
     entirety, the list will of course consist of just the tree's root node.
-
     """
 
     current_path = []
@@ -247,7 +257,15 @@ def cache_tree_children(queryset):
         mptt_opts = queryset.model._mptt_meta
         tree_id_attr = mptt_opts.tree_id_attr
         left_attr = mptt_opts.left_attr
-        queryset = queryset.order_by(tree_id_attr, left_attr)
+        if tuple(queryset.query.order_by) != (tree_id_attr, left_attr):
+            warnings.warn(
+                (
+                    "cache_tree_children() was passed a queryset with the wrong ordering: %r.\n"
+                    "This will cause an error in mptt 0.8."
+                ) % (queryset.query.order_by,),
+                UserWarning,
+            )
+            queryset = queryset.order_by(tree_id_attr, left_attr)
 
     if queryset:
         # Get the model's parent-attribute name
@@ -286,6 +304,10 @@ def cache_tree_children(queryset):
                 setattr(obj, parent_attr, _parent)
                 _parent._cached_children.append(obj)
 
+                if root_level == 0:
+                    # get_ancestors() can use .parent.parent.parent...
+                    setattr(obj, '_mptt_use_cached_ancestors', True)
+
             # Add the current node to end of the current path - the last node
             # in the current path is the parent for the next iteration, unless
             # the next iteration is higher up the tree (a new branch), in which
